/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

package at.spardat.xma.component;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.ArrayList;



/**
 * This class makes it possible to fully integrate components of different server side applications
 * in one client side application. It is used internally by the runtime.
 * 
 * @author s2877
 */
public class ProxyHandler implements InvocationHandler {
    /** the object which implements the interfaces */
    Object impl;
    
    /**
     * Creates a ProxyHandler for a given object.
     * @param obj The object for which a proxy is needed
     */
    public ProxyHandler(Object obj) {
        impl=obj;    
    }

    /* (non-Javadoc)
     * @see java.lang.reflect.InvocationHandler#invoke(java.lang.Object, java.lang.reflect.Method, java.lang.Object[])
     */
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        try {
            
            // handle equals: call it allways with the original object
            if(method.getName().equals("equals")) {
                Class[] paramTypes = method.getParameterTypes();
                if(paramTypes.length==1&&paramTypes[0].equals(Object.class)) {
                    if(Proxy.isProxyClass(args[0].getClass())) {
                        Object other = Proxy.getInvocationHandler(args[0]);
                        if(other instanceof ProxyHandler) {
                            return method.invoke(impl,new Object[]{((ProxyHandler)other).impl});                            
                        }
                    }
                }
            }
            
            // handle all other methods
            Class[] paramTypes = method.getParameterTypes();
            for(int i=0;i<paramTypes.length;i++) {
                if(!paramTypes[i].isPrimitive()) {
                    paramTypes[i] = impl.getClass().getClassLoader().loadClass(paramTypes[i].getName());
                    args[i] = bridgeClassLoaders(impl.getClass().getClassLoader(),args[i]);
                }
            }
            Method implMethod = impl.getClass().getMethod(method.getName(),paramTypes);
            Object result = implMethod.invoke(impl,args);
            return bridgeClassLoaders(this.getClass().getClassLoader(),result);
            
        } catch (InvocationTargetException exc) {
            throw exc.getTargetException();
        }
    }
    

    /**
     * @param interfaceLoader the classloader that loaded the interfaces which will be
     *          used to access the object.
     * @param object the object to access
     */
    static public Object bridgeClassLoaders(ClassLoader interfaceLoader,Object object) {
        // if the object is loaded by the same classloader, no proxy is needed.
        if(object==null || interfaceLoader.equals(object.getClass().getClassLoader())) {
            return object;
        }
        // prevent accumulation of proxies
        for(Object o=object;Proxy.isProxyClass(o.getClass());) {
            Object handler = Proxy.getInvocationHandler(o);
            if(!(handler instanceof ProxyHandler)) break;
            o=((ProxyHandler)handler).impl;
            if(interfaceLoader.equals(o.getClass().getClassLoader())) {
                return o;
            }
        }
        
        // check if there are any interfaces loades by differnet classloaders
        boolean needProxy = false;
        ArrayList ints = new ArrayList();
        for(Class objClass = object.getClass();objClass!=null;objClass = objClass.getSuperclass()) {
            Class[] interfaces = objClass.getInterfaces();
            for(int i=0;i<interfaces.length;i++) {
                try {
                    Class sourceClass = interfaceLoader.loadClass(interfaces[i].getName());
                    if(sourceClass.equals(interfaces[i])) {
                        if(!ints.contains(interfaces[i])) {
                            ints.add(interfaces[i]);
                        }
                    } else {
                        if(!ints.contains(sourceClass)) {
                            ints.add(sourceClass);
                            needProxy = true;
                        }
                    }
                } catch (ClassNotFoundException e) {
                    // If the interface is not known in the source classloader,
                    // the object just do not implent it -> nothing to do.
                }   
            }
        }
        if(needProxy) {
            Class[] interfaces = new Class[ints.size()];
            for(int i=0;i<interfaces.length;i++) {
                interfaces[i] = (Class) ints.get(i);
            }
            return Proxy.newProxyInstance(interfaceLoader,interfaces,new ProxyHandler(object));
        } else {
            return object;
        }
    }
}
