/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * @(#) $Id: HashFilter.java 2089 2007-11-28 13:56:13Z s3460 $
 *
 * 
 * 
 * 
 *
 */
package at.spardat.xma.session;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

/**
 * Filters hash values from the URL in the request. A hash value in an URL is defined
 * by the following regular expression: <code> \.[0-9a-f]{32}\. </code>. It consists of
 * 32 hex digits beginning and ending with a dot. It is inserted into the URL by the
 * boot runtime to avoid problems with proxies probably caching wrong versions of the requested
 * file.
 * 
 * @author s2877  
 * @since 1.6.0
 */
public class HashFilter implements Filter {
    private static final int hashLength=32;
    private FilterConfig config;

    /**
     * The doFilter method of the Filter is called by the container each time a request/response pair is
     * passed through the chain due to a client request for a resource at the end of the chain.
     * A wrapper is applied to the request and passed on to the next filter or servlet down the chain.
     * This wrapper overriedes all methods returning the request url or parts of it and filters an
     * eventuall hash value. 
     */
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        ServletRequest wrapper = new HttpServletRequestWrapper((HttpServletRequest)request) {
            public String getPathInfo() {
                return filterHash(super.getPathInfo());
            }
            public String getPathTranslated() {
                return filterHash(super.getPathTranslated());
            }
            public String getRequestURI() {
                return filterHash(super.getRequestURI());
            }
            public StringBuffer getRequestURL() {
                return new StringBuffer(filterHash(super.getRequestURL().toString()));
            }
            public String getServletPath() {
                return filterHash(super.getServletPath());
            }
            public String getRealPath(String arg0) {
                return filterHash(super.getRealPath(arg0));
            }
        };
        chain.doFilter(wrapper,response);
    }
    
    /**
     * Part of Filter Interface contained in weblogic 6.1
     * @return the FilterConfig set by setFilterConfig().
     */
    public FilterConfig getFilterConfig() {
        return config;
    }

    /**
     * Called by weblogic 6.1 instead of init().
     */
    public void setFilterConfig(FilterConfig config) {
        this.config=config;
    }

    /**
     * Called by Servlet 2.3 complient web containers to indicate to a filter that it is being placed into service. 
     * The servlet container calls the init method exactly once after instantiating the filter. 
     */
    public void init(FilterConfig config) throws ServletException {
        setFilterConfig(config);
    }

    /**
     * Called by Servlet 2.3 complient web containers to indicate to a filter that it is being taken out of service. 
     * This method is only called once all threads within the filter's doFilter method have exited 
     * or after a timeout period has passed. After the web container calls this method, it will not 
     * call the doFilter method again on this instance of the filter.
     */
    public void destroy() {}   
    
    /**
     * Filters hash values from the given string. A hash value is defined
     * by the following regular expression: <code> \.[0-9a-f]{32}\. </code>. It consists of
     * 32 hex digits beginning and ending with a dot.
     * @param name the url or part of an url to filter
     * @return name with the hash-part removed.
     */
    public String filterHash(String name) {
        if(name==null) return name;
        int hashStart=0;
        int counter=0;
        boolean inHash=false;
        for(int i=0;i<name.length();i++) {
            char c=name.charAt(i);
            if(c=='.') {
                if(inHash) {
                   if(counter==hashLength) {
                       return name.substring(0,hashStart)+name.substring(hashStart+hashLength+1,name.length());                       
                   }
                }
                hashStart=i;
                counter=0;
                inHash=true;
            } else {
                if(c>='0'&&c<='9' || c>='a'&&c<='f') {
                    if(inHash) counter++;
                } else {
                    inHash=false;
                }
            }
        }
        if(counter==32) { // hash at the end
            return name.substring(0,hashStart);
        } else { //no hash found
            return name;
        }
    }

}
